// Copyright (C) 2000 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// As a special exception to the GNU General Public License, permission is
// granted for additional uses of the text contained in its release
// of Bayonne as noted here.
//
// This exception is that permission is hereby granted to link Bayonne 
// with  the Pika MonteCarlo libraries to produce a executable image
// without requiring MonteCarlo itself to be supplied in source form so
// long as each source file so linked contains this exclusion.
//
// This exception does not however invalidate any other reasons why
// the resulting executable file might be covered by the GNU General
// public license or invalidate the licensing requirements of any
// other component or library.
//
// This exception applies only to the code released by OST under the
// name Bayonne.  If you copy code from other releases into a copy of
// Bayonne, as the General Public License permits, the exception does not
// apply to the code that you add in this way.  To avoid misleading
// anyone as to the status of such modified files, you must delete
// this exception notice from them.
//
// If you write modifications of your own to Bayonne, it is your choice
// whether to permit this exception to apply to your modifications.
// If you do not wish that, delete this exception notice, at which
// point the terms of your modification would be covered under the GPL
// as explicitly stated in "COPYING".

#include <ivrconfig.h>
#include <bayonne.h>
#include <cc++/audio.h>

struct vpbreg;

class VPBTrunk;

class VPBConfig : public Keydata
{
public:
	VPBConfig();

	int getFirst(void)
		{return strtol(getLast("first"), NULL, 16);};

	int getCards(void)
		{return atoi(getLast("cards"));};

	int getChannels(void)
		{return atoi(getLast("channels"));};

	int getMsgSize(void)
		{return atoi(getLast("messagesize"));};

	int getRxSize(void)
		{return atoi(getLast("receivesize"));};

	int getTxSize(void)
		{return atoi(getLast("transmitsize"));};

	int getFrameSize(void)
		{return atoi(getLast("framesize"));};

	int getRelaySize(void)
		{return atoi(getLast("relaysize"));};

	char *getFirmware(void)
		{return getLast("firmware");};
	
};

class VPBThread : public Semaphore, public Mutex, public Thread
{
private:
	friend class VPBDriver;
	friend class VPBTrunk;

	static int offset;
	static int cards;
	static int count;
	void *reg;

	unsigned char control;	// control register shadow
	unsigned port;		// port address of card
	unsigned short pipmask;
	bool active;
	int id;

	unsigned short model, ddmodel, szRelayBuf;

	void Run(void);

	void setControl(unsigned char flags);
	void clrControl(unsigned char flags);
	void dspWrite(unsigned short addr, unsigned short *data, unsigned len);
	void dspRead(unsigned short addr, unsigned short *data, unsigned len);
	void dspReset(void);
	void dspEnable(void);
	bool dspTest(unsigned size);
	vpbreg *getRegistry(void);

	inline void setPip(unsigned char enable)
		{pipmask = enable << 8;};

	VPBThread();
	~VPBThread();
};

class VPBDriver : public Driver, public VPBConfig
{
private:
	VPBThread *threads;
	VPBTrunk **ports;
	int port_count, thread_count;

public:
	VPBDriver();
	~VPBDriver();

	int Start(void);
	void Stop(void);

	int getTrunkCount(void)
		{return port_count;};

	Trunk *getTrunkPort(int id);
	aaScript *getScript(void);
};

extern VPBDriver vpbivr;
