// Copyright (C) 2000 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// As a special exception to the GNU General Public License, permission is
// granted for additional uses of the text contained in its release
// of Bayonne as noted here.
//
// This exception is that permission is hereby granted to link Bayonne 
// with  the Pika MonteCarlo libraries to produce a executable image
// without requiring MonteCarlo itself to be supplied in source form so
// long as each source file so linked contains this exclusion.
//
// This exception does not however invalidate any other reasons why
// the resulting executable file might be covered by the GNU General
// public license or invalidate the licensing requirements of any
// other component or library.
//
// This exception applies only to the code released by OST under the
// name Bayonne.  If you copy code from other releases into a copy of
// Bayonne, as the General Public License permits, the exception does not
// apply to the code that you add in this way.  To avoid misleading
// anyone as to the status of such modified files, you must delete
// this exception notice from them.
//
// If you write modifications of your own to Bayonne, it is your choice
// whether to permit this exception to apply to your modifications.
// If you do not wish that, delete this exception notice, at which
// point the terms of your modification would be covered under the GPL
// as explicitly stated in "COPYING".

#include "server.h"
#include <ctype.h>

FifoHandler *FifoHandler::first = NULL;
Fifo fifo;

static Trunk *getTrunk(char *str)
{
	if(!str)
		return NULL;

	return driver->getTrunkPort(atoi(str));
}

bool Fifo::waitPid(char **argv)
{
	Trunk *trunk = getTrunk(argv[1]);
	if(!trunk)
		return false;

	if(!argv[2])
		return false;

	trunk->tgi.pid = atoi(argv[2]);
	return true;
}

bool Fifo::setSymbol(char **argv)
{
	Trunk *trunk = getTrunk(argv[1]);
	if(!trunk)
		return false;

	if(!argv[3] || !argv[2])
		return false;

	trunk->setSymbol(argv[2], trunk->getSymbolSize());
	trunk->setSymbol(argv[2], argv[3]);
	return true;
}

bool Fifo::login(char **argv)
{
	Trunk *trunk = getTrunk(argv[1]);
	TrunkEvent event;
	char *buffer;

	if(!trunk || !argv[2])
		return false;

	if(!database)
		return false;

	trunk->setSymbol("login", database->getLoginSize());
	buffer = trunk->getSymbol("login");
	if(buffer)
		if(*buffer)
			trunk->Logout();

	trunk->setSymbol("login", "");
	event.parm.lookup.data = trunk->getSymbol("login");
	event.parm.lookup.seq = atoi(argv[2]);
	event.parm.lookup.result = true;
	event.id = TRUNK_SERVICE_LOGIN;
	if(trunk->postEvent(&event))
	{
		trunk->setSymbol("login", argv[3]);
		return true;
	}
	return false;
}	

bool Fifo::submit(char **argv)
{
	Trunk *trunk = getTrunk(argv[1]);
	char buffer[512];
	strstream str(buffer, 512);
	TrunkEvent event;
	int argc = 3;
	unsigned seq;

	if(!trunk || !argv[2])
		return false;

	seq = atoi(argv[2]);
	while(argv[argc])
	{
		if(argc > 3)
			str << '&';
		str << argv[argc++];
	}
	str << ends;
	event.id = TRUNK_SERVICE_LOOKUP;
	event.parm.lookup.seq = seq;
	event.parm.lookup.result = true;
	event.parm.lookup.data = buffer;
	return trunk->postEvent(&event);
}


bool Fifo::ringScript(int argc, char **argv)
{
	Trunk *trunk = getTrunk(argv[1]);
	TrunkEvent event;

	if(!trunk || !argv[2])
		return false;

	event.id = TRUNK_RING_START;
	event.parm.argv = &argv[2];

	return trunk->postEvent(&event);
}

bool Fifo::startScript(int argc, char **argv)
{
	Trunk *trunk = getTrunk(argv[1]);	
	TrunkEvent event;
	
	if(!trunk || !argv[2])
		return false;

	event.id = TRUNK_START_SCRIPT;
	event.parm.argv = &argv[2];

	return trunk->postEvent(&event);
}

bool Fifo::exitPid(char **argv)
{
	Trunk *trunk = getTrunk(argv[1]);
	TrunkEvent event;

	if(!trunk)
		return false;

	event.id = TRUNK_EXIT_SHELL;
	if(argv[2])
		event.parm.status = atoi(argv[2]);
	else
		event.parm.status = 0;

	return trunk->postEvent(&event);
}
		
bool Fifo::Command(const char *cmd)
{
	FifoHandler *next;
	char buffer[512];
	bool rts = false;
	char *argv[65];
	int argc = 0;
	char *arg;
	char *sp;
	char *ptr;
	const char *token = keyserver.getToken();
	int tlen = strlen(token);

	strncpy(buffer, cmd, sizeof(buffer) - 1);
	buffer[sizeof(buffer) - 1] = 0;

	EnterMutex();
	slog(SLOG_DEBUG) << "fifo: cmd=" << cmd << endl;
	if(strstr(buffer, token))
	{
		ptr = buffer;
		while(isspace(*ptr))
			++ptr;
	
		ptr = strtok_r(ptr, "\n", &sp);	
		while(NULL != (sp = strstr(ptr, token)))
		{
			argv[argc++] = ptr;
			*sp = 0;
			ptr = sp + tlen;
		}
		while(isspace(*ptr))
			++ptr;
		if(*ptr)
			argv[argc++] = ptr;
	}
	else
	{
		argv[argc++] = strtok_r(buffer, " \t\n\r", &sp);
		while(argc < 64)
		{
			arg = strtok_r(NULL, " \t\n\r", &sp);
			if(!arg)
				break;
			argv[argc++] = arg;
		}
	}
	argv[argc] = NULL;

	if(!argv[0])
	{
		LeaveMutex();
		return false;
	}

	if(!*argv[0])
	{
		LeaveMutex();
		return false;
	}

	next = FifoHandler::first;

	while(next && !rts)
	{
		rts = next->Command(argv);
		next = next->next;
	}

	if(!stricmp(argv[0], "down"))
	{
		raise(SIGINT);
		rts = true;
	}
	else if(!stricmp(argv[0], "compile"))
	{
		driver->getImage();
		rts = true;
	}		
	else if(!stricmp(argv[0], "wait"))
		rts = waitPid(argv);
	else if(!stricmp(argv[0], "exit"))
		rts = exitPid(argv);
	else if(!stricmp(argv[0], "set"))
		rts = setSymbol(argv);
	else if(!stricmp(argv[0], "submit"))
		rts = submit(argv);
	else if(!stricmp(argv[0], "login"))
		rts = login(argv);
	else if(!stricmp(argv[0], "debug"))
		rts = debug->DebugFifo(argv);
	else if(!stricmp(argv[0], "ring"))
		rts = ringScript(argc, argv);

//  MattB

	else if(!stricmp(argv[0], "start"))
		rts = startScript(argc, argv);	

	LeaveMutex();
	return rts;
}

FifoHandler::FifoHandler()
{
	next = first;
	first = this;
}

